#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

int
convolve2(double **y, unsigned *ylen, const double *x, unsigned xlen, const double *h, unsigned hlen)
{
	double *yout;
	unsigned n, size;
	int i, j, idx;

	n = xlen + hlen - 1;
	size = sizeof(double) * n;

	yout = malloc(size);

	if (yout == NULL)
		return -1;

	for (i = 0; i < n; i++)
		yout[i] = 0;

	for (i = 0; i < n; i++) {
		for (j = 0; j < hlen; j++) {
			idx = i - j;
			if (idx < 0 || idx >= n)
				break;
			yout[i] += h[j] * x[idx];
		}
	}

	*ylen = n;
	*y = yout;

	return 0;
}

int
convolve(double **y, unsigned *ylen, const double *x, unsigned xlen, const double *h, unsigned hlen)
{
	double *yout;
	unsigned n, size;
	int i, j;

	n = xlen + hlen - 1;
	size = sizeof(double) * n;

	yout = malloc(size);

	if (yout == NULL)
		return -1;

	for (i = 0; i < n; i++)
		yout[i] = 0;

	for (i = 0; i < xlen; i++) {
		for (j = 0; j < hlen; j++) {
			yout[i+j] += h[j] * x[i];
		}
	}

	*ylen = n;
	*y = yout;

	return 0;
}

int
main(int argc, char *argv[])
{
	int i;
	double *y;
	unsigned ylen;
	double h[30] = {
		0.0, 0.01, 0.015, 0.02, 0.025,
		0.030, 0.034, 0.038, 0.040, 0.042,
		0.0435, 0.0440, 0.0443, 0.045, 0.0455,
		0.0455, 0.045, 0.0443, 0.0440, 0.0435,
		0.042, 0.040, 0.038, 0.034, 0.030,
		0.025, 0.02, 0.015, 0.01, 0.0
	};
	double x[100];
	double quantum, t;
	FILE *xfile, *yfile;

	xfile = fopen("xfile", "w");
	
	if (xfile == NULL)
		exit(1);

	yfile = fopen("yfile", "w");

	if (yfile == NULL)
		exit(1);
	
	t = 0.0;
	quantum = 0.1;

	for (i = 0; i < 100; i++) {
		x[i] = sin(5.0*t) + 1.5*t;	
		fprintf(xfile, "%f %f\n", t, x[i]);
		t += quantum;
	}

	convolve2(&y, &ylen, x, 100, h, 30);

	t = 0.0;

	for (i = 0; i < ylen; i++) {
		fprintf(yfile, "%f %f\n", t, y[i]);
		t += quantum;
	}

	fclose(xfile);
	fclose(yfile);

	return 0;
}
